"""Define interfaces for your add-on.
"""
from zope.interface import Interface
from zope import schema

from zope.i18nmessageid import MessageFactory
from plone.theme.interfaces import IDefaultPloneLayer
from browser.interfaces import *

_ = MessageFactory('collective.flexitopic')

class IFlexiTopicLayer(IDefaultPloneLayer):
    """Marker interface that defines a Zope 3 browser layer.
    """

class IFlexiTopicSettings(Interface):
    """Global settings. This describes records stored in the
    configuration registry and obtainable via plone.registry.
    """

    flexitopic_width = schema.Int(title=_(u"Flexitopic width"),
                                  description=_(u"Width of the flexigrid table"),
                                  required=True,
                                  default=900)

    flexitopic_height = schema.Int(title=_(u"Flexitopic height"),
                                  description=_(u"Height of the flexigrid table"),
                                  required=True,
                                  default=200)
    items_pp = schema.Choice(
         title=_(u'Items per page'),
         description=_(u"Number of items displayed in the table"),
         values=(10, 15, 20, 30, 50),
         default=20,
         required=True,
    )



from plone.app.registry.browser import controlpanel

from collective.flexitopic.interfaces import IFlexiTopicSettings, _


class FlexiTopicEditForm(controlpanel.RegistryEditForm):

    schema = IFlexiTopicSettings
    label = _(u"Flexitopic settings")
    description = _(u"Flexitopic settings")

    def updateFields(self):
        super(FlexiTopicEditForm, self).updateFields()


    def updateWidgets(self):
        super(FlexiTopicEditForm, self).updateWidgets()

class FlexiTopicControlPanel(controlpanel.ControlPanelFormWrapper):
    form = FlexiTopicEditForm



  # -*- extra stuff goes here -*- 
from zope.i18nmessageid import MessageFactory

flexitopicMessageFactory = MessageFactory('collective.flexitopic')

def initialize(context):
    """Initializer called when used as a Zope 2 product."""



from zope.interface import implements
from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName

from interfaces import IFlexiTopicView
from utils import get_renderd_table

class FlexiTopicView(BrowserView):

    implements(IFlexiTopicView)

    __call__ = ViewPageTemplateFile('templates/flexitopicview.pt')

    def __init__(self, context, request):
        self.context = context
        self.request = request

    @property
    def portal_catalog(self):
        return getToolByName(self.context, 'portal_catalog')

    @property
    def portal(self):
        return getToolByName(self.context, 'portal_url').getPortalObject()




class FlexiCollectionView(FlexiTopicView):
    """
    FlexiTopic browser view
    """
    __call__ = ViewPageTemplateFile('templates/flexicollectionview.pt')



from zope.interface import Interface
from zope.viewlet.interfaces import IViewletManager


class IFlexiTopicView(Interface):
    """
    FlexiTopic view interface
    """

class IFlexiJsonView(Interface):
    """
    FlexiJson view interface
    """

# viewletmanger interfaces for FlexiTopicView

class IFlexiTopicTop(IViewletManager):
    ''' The top of a flexitopic page '''

class IFlexiTopicForm(IViewletManager):
    ''' Render the seach form '''

class IFlexiTopicFormExt(IViewletManager):
    ''' insert into the seach form '''


class IFlexiTopicBottom(IViewletManager):
    ''' The botom of a flexitopic page
       (the results table)
    '''
class IFlexiTopicJs(IViewletManager):
    '''Slot to insert the JS into a flexitopic page '''




#utils
import logging
from time import time
from DateTime import DateTime
from zope.component import getUtility
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone.memoize import ram
try:
    from plone.app.querystring import queryparser
except ImportError:
    pass
from plone.registry.interfaces import IRegistry

from collective.flexitopic.interfaces import IFlexiTopicSettings
from collective.flexitopic import flexitopicMessageFactory as _
try:
    from plone.app.querystring import queryparser
except ImportError:
    pass

logger = logging.getLogger('collective.flexitopic')

logger.setLevel(logging.DEBUG)

# just to cheat i18ndude
title = _('Title')
phone = _('Phone')

RAM_CACHE_SECONDS = 60

DATERANGE_OPERATORS = [
    'plone.app.querystring.operation.date.largerThanRelativeDate', #within next n days
    'plone.app.querystring.operation.date.beforeToday',
    'plone.app.querystring.operation.date.lessThanRelativeDate', #within last n days
    'plone.app.querystring.operation.date.largerThan', #after date
    #'plone.app.querystring.operation.date.today',
    'plone.app.querystring.operation.date.afterToday',
    'plone.app.querystring.operation.date.lessThan', # before date
    'plone.app.querystring.operation.date.between', # between date and date
    ]

IDX_METADATA = {
        'Title': 'sortable_title',
        'ExpirationDate': 'expires',
        'ModificationDate': 'modified',
        'EffectiveDate': 'effective',
        'CreationDate': 'created'}

def _datelimit_cachekey(fun, context, criterion, portal_catalog):
    query = context.buildQuery()
    query.pop(criterion.Field(),None)
    ckey = list(context.getPhysicalPath())
    for dcriterion in context.listCriteria():
        if dcriterion.meta_type in ['ATFriendlyDateCriteria']:
            query.pop(criterion.Field(),None)
    ckey.append(query)
    ckey.append(criterion.Field())
    ckey.append(time() // RAM_CACHE_SECONDS)
    return ckey

@ram.cache(_datelimit_cachekey)
def get_date_limit(context, criterion, portal_catalog):
    '''
    get the earliest/latest date that should be included in
    a daterange covering all dates defined by the criteria
    '''
    query = context.buildQuery()
    query['sort_on'] = criterion.Field()
    query['sort_limit'] = 1
    if criterion.getOperation() == 'less':
        query['sort_order'] = 'ascending'
        adjust = -1
    else:
        query['sort_order'] = 'descending'
        adjust = 1
    query.pop(criterion.Field())
    results = portal_catalog(**query)
    if len(results) > 0:
        date_limit = results[0][criterion.Field()] + adjust
    else:
        date_limit = DateTime()
    return date_limit

def get_start_end(flexitopic, criterion, catalog):
    #for old style collections
    if criterion.meta_type in ['ATDateRangeCriterion']:
        start_date = criterion.getStart()
        end_date = criterion.getEnd()
    elif criterion.meta_type in ['ATFriendlyDateCriteria']:
        date_base = criterion.getCriteriaItems()[0][1]['query']
        if date_base==None:
            date_base = DateTime()
        date_limit = get_date_limit(flexitopic.context, criterion,
                                    catalog)
        start_date = min(date_base, date_limit)
        end_date =  max(date_base, date_limit)
    return start_date, end_date


def get_first_date(criterion, query, catalog, sort_order):
    query['sort_on'] = criterion
    query['sort_limit'] = 1
    query['sort_order'] = sort_order
    results = catalog(**query)
    if len(results) > 0:
        date_limit = results[0][criterion] #+ adjust
    else:
        date_limit = DateTime()
    return date_limit

def get_max_date(criterion, query, catalog):
    return get_first_date(criterion, query, catalog, 'descending')


def get_min_date(criterion, query, catalog):
    return get_first_date(criterion, query, catalog, 'ascending')



def get_start_end_ng(context, query, raw, catalog):
    if raw['o']=='plone.app.querystring.operation.date.largerThanRelativeDate':
        #within last n days
        return query[raw['i']]['query'][0], query[raw['i']]['query'][1]
    elif raw['o']=='plone.app.querystring.operation.date.beforeToday':
        return get_min_date(raw['i'], query, catalog), DateTime()
    elif raw['o']=='plone.app.querystring.operation.date.lessThanRelativeDate':
        #within next n days
        return query[raw['i']]['query'][0], query[raw['i']]['query'][1]
    elif raw['o']=='plone.app.querystring.operation.date.largerThan':
        #after date
        return DateTime(raw['v']), get_max_date(raw['i'], query, catalog)
    elif raw['o']=='plone.app.querystring.operation.date.afterToday':
        return DateTime(), get_max_date(raw['i'], query, catalog)
    elif raw['o']=='plone.app.querystring.operation.date.lessThan':
        # before date
        return get_min_date(raw['i'], query, catalog), DateTime(raw['v'])
    elif raw['o']=='plone.app.querystring.operation.date.between':
        # between date and date
        return DateTime(raw['v'][0]), DateTime(raw['v'][1])
    else:
        logger.error('operation %s not supported' % raw['o'])




def get_renderd_table(context, search_results):
    """ Render results table

    @return: Resulting HTML code as Python string
    """
    table_template = ViewPageTemplateFile("templates/table.pt")
    return table_template(context, search_results=search_results)

def get_topic_table_fields(context, catalog):
    fields = context.getCustomViewFields()
    registry = getUtility(IRegistry)
    settings = registry.forInterface(IFlexiTopicSettings)
    field_list =[]
    vocab = context.listMetaDataFields(False)
    col_width = int(settings.flexitopic_width/len(fields))
    for field in fields:
        if field in IDX_METADATA.keys():
            idx = catalog.Indexes[IDX_METADATA[field]].meta_type
        elif field in catalog.Indexes.keys():
            idx = catalog.Indexes[field].meta_type
        else:
            idx = None
        name = vocab.getValue(field, field)
        field_list.append({'name': field, 'label': name, 'idx_type': idx,
                            'col_width': col_width})
    return field_list


def _search_result_cachekey(fun, flexitopic):
    ckey = [flexitopic.request.form]
    ckey.append(flexitopic.context.getPhysicalPath())
    ckey.append(time() // RAM_CACHE_SECONDS)
    return ckey


@ram.cache(_search_result_cachekey)
def get_search_results(flexitopic):
    form = flexitopic.request.form

    batch_size = form.get('b_size', 20)
    batch_start = form.get('b_start', 0)
    catalog = flexitopic.portal_catalog
    query = flexitopic.context.buildQuery()
    for criterion in flexitopic.context.listCriteria():
        if criterion.meta_type in ['ATDateRangeCriterion',
                                    'ATFriendlyDateCriteria']:
            start_date, end_date = get_start_end(flexitopic, criterion, catalog)
            value = (form.get('start-' + criterion.Field(),
                            start_date.strftime('%Y/%m/%d')),
                    form.get('end-' + criterion.Field(),
                            end_date.strftime('%Y/%m/%d')))
        else:
            value = form.get(criterion.Field(),None)
        if value:
            query[criterion.Field()] = {}
            if hasattr(criterion, 'getOperator'):
                operator = criterion.getOperator()
                query[criterion.Field()]['operator'] = operator
                assert(criterion.meta_type in
                        ['ATSelectionCriterion',
                        'ATListCriterion'])
            else:
                operator = None
            if operator =='or':
                query[criterion.Field()] = { 'query':[value],
                    'operator':'or'}
            elif operator == 'and':
                q = list(criterion.Value()) + [value]
                query[criterion.Field()] = { 'query':[q],
                    'operator':'and'}
            else:
                if criterion.meta_type in ['ATDateRangeCriterion',
                            'ATFriendlyDateCriteria']:
                    start = DateTime(value[0])
                    end = DateTime(value[1])
                    query[criterion.Field()] = {'query':(start, end),
                                'range': 'min:max'}

                else:
                    assert(criterion.meta_type=='ATSimpleStringCriterion')
                    if criterion.Value():
                        query[criterion.Field()] =  { 'query':
                                    [criterion.Value(), value],
                                    'operator':'and'}
                    else:
                        query[criterion.Field()] = value
        else:
            continue

    sortorder = form.get('sortorder',None)

    if sortorder=='desc':
        sort_order = 'reverse'
    else:
        sort_order = None
    sort_on = None
    sortname = form.get('sortname',None)
    if sortname in IDX_METADATA.keys():
        sort_on = IDX_METADATA[sortname]
    elif sortname in catalog.Indexes.keys():
        if catalog.Indexes[sortname].meta_type in [
                'FieldIndex', 'DateIndex', 'KeywordIndex']:
            sort_on = sortname
    elif sortname == None:
        #get sort_on/order out of topic
        for criterion in flexitopic.context.listCriteria():
            if criterion.meta_type =='ATSortCriterion':
                sort_on = criterion.getCriteriaItems()[0][1]
                if len(criterion.getCriteriaItems())==2 and sortorder==None:
                    sort_order = criterion.getCriteriaItems()[1][1]
    if sort_on:
        query['sort_on'] = sort_on
        if sort_order:
            query['sort_order'] = sort_order

    logger.debug(query)
    results = catalog(**query)

    return {'results': results, 'size': batch_size,
        'start': batch_start, 'num_results': len(results)}

def get_search_results_ng(flexitopic):
    form = flexitopic.request.form

    batch_size = form.get('b_size', 20)
    batch_start = form.get('b_start', 0)
    catalog = flexitopic.portal_catalog
    query = queryparser.parseFormquery(flexitopic.context,
            flexitopic.context.getRawQuery())
    for raw_query in flexitopic.context.getRawQuery():
            value = form.get(raw_query['i'], False)
            if value:
                if (
                (raw_query['o'] == 'plone.app.querystring.operation.selection.is')
                and (value in raw_query['v'])):
                    query[raw_query['i']]['query'] = value
                elif raw_query['o'] == 'plone.app.querystring.operation.string.contains':
                    if raw_query['v']:
                        query[raw_query['i']]['query'] = value + ' AND ' + raw_query['v']
                    else:
                        query[raw_query['i']]['query'] = value
            elif raw_query['o'] in DATERANGE_OPERATORS:
                start = form.get('start-' + raw_query['i'], False)
                end = form.get('end-' + raw_query['i'], False)
                if start and end:
                    query[raw_query['i']] = {'query':(start, end),
                                'range': 'min:max'}


    sortorder = form.get('sortorder',None)

    if sortorder=='desc':
        sort_order = 'reverse'
    else:
        sort_order = None
    sort_on = None
    sortname = form.get('sortname',None)
    if sortname in IDX_METADATA.keys():
        sort_on = IDX_METADATA[sortname]
    elif sortname in catalog.Indexes.keys():
        if catalog.Indexes[sortname].meta_type in [
                'FieldIndex', 'DateIndex', 'KeywordIndex']:
            sort_on = sortname
    elif sortname == None:
        #get sort_on/order out of topic
        sort_on = flexitopic.context.getSort_on()
        if flexitopic.context.getSort_reversed():
            sort_order = 'reverse'
    if sort_on:
        query['sort_on'] = sort_on
        if sort_order:
            query['sort_order'] = sort_order
    logger.debug(query)
    results = catalog(**query)
    return {'results': results, 'size': batch_size,
        'start': batch_start, 'num_results': len(results)}



# -*- coding: utf-8 -*-
import logging
from Products.CMFCore.utils import getToolByName

# The profile id of your package:
PROFILE_ID = 'profile-collective.flexitopic:default'

def add_ng_collection(context, logger=None):
    if logger is None:
        # Called as upgrade step: define our own logger.
        logger = logging.getLogger('collective.flexitopic')
    logger.info("Add typeinfo for new style collections")
    setup = getToolByName(context, 'portal_setup')
    setup.runImportStepFromProfile(PROFILE_ID, 'typeinfo')

def upgrade_registry(context, logger=None):
    """Re-import the portal configuration registry settings.
    """
    if logger is None:
        # Called as upgrade step: define our own logger.
        logger = logging.getLogger('collective.flexitopic')
    logger.info("Update registry")
    setup = getToolByName(context, 'portal_setup')
    setup.runImportStepFromProfile(PROFILE_ID, 'plone.app.registry')
    return

def update_js(context, logger=None):
    """Re-import the portal javascript settings.
    """
    if logger is None:
        # Called as upgrade step: define our own logger.
        logger = logging.getLogger('collective.flexitopic')
    logger.info("Update javascripts")
    setup = getToolByName(context, 'portal_setup')
    setup.runImportStepFromProfile(PROFILE_ID, 'jsregistry')
    return



from types import StringTypes

from zope.interface import implements
from zope.component import getMultiAdapter
from interfaces import IFlexiJsonView
import logging

from Products.Five import BrowserView
from Products.CMFCore.utils import getToolByName

try:
    import simplejson as json
except ImportError:
    import json


from utils import get_search_results, get_topic_table_fields, get_search_results_ng

logger = logging.getLogger('collective.flexitopic')




class FlexiJsonView(BrowserView):
    """
    FlexiJson browser view
    """
    implements(IFlexiJsonView)

    def __init__(self, context, request):
        self.context = context
        self.request = request

    @property
    def site_properties(self):
        return getToolByName(self.context, 'portal_properties').site_properties

    @property
    def portal_catalog(self):
        return getToolByName(self.context, 'portal_catalog')

    @property
    def portal(self):
        return getToolByName(self.context, 'portal_url').getPortalObject()

    def _get_search_results(self):
        return get_search_results(self)['results']

    def __call__(self):
        """
        Return JSON for flexigrid, the query form looks like:
        {'getAgencies': '', 'rp': '15', 'sortname': 'Title', 'Title': '',
        'getProject_status': '', 'getSubRegions': '', 'SearchableText': '',
        'getProject_type': '', 'sortorder': 'asc', 'query': '',
        'getBasin': '', 'qtype': 'getSubRegions', 'page': '1'}
        """
        util = getToolByName(self.context, 'translation_service')
        form = self.request.form
        limit = int(form.get('rp', '15'))
        start = (int(form.get('page', '1')) - 1) * limit
        end = start + limit + 1
        results = self._get_search_results()
        json_result= {"page":form.get('page', '1') ,
            "total":len(results),
            "rows":[]}
        fields = get_topic_table_fields(self.context, self.portal_catalog)
        layout = getMultiAdapter((self.context, self.request), name=u'plone_layout')
        use_view_action = self.site_properties.typesUseViewActionInListings
        for result in results[start:end]:
            cell = []
            icon = layout.getIcon(result)
            if icon.url:
                icon_snp = u'<img src="%s" alt="%s" title="%s" width="%i" height="%i" />'%(
                    icon.url, icon.description, icon.title, icon.width, icon.height)
            else:
                icon_snp = ''
            for field in fields:
                value = getattr(result, field['name'])
                if type(value)==tuple:
                    vt =[]
                    for v in value:
                        v = v.decode('utf-8', 'ignore').encode('ascii', 'xmlcharrefreplace')
                        vt.append(v)
                    value = vt
                elif type(value) in StringTypes:
                    value = value.decode('utf-8', 'ignore').encode('ascii', 'xmlcharrefreplace')
                if not value:
                    cell.append('')
                elif field['idx_type'] == 'DateIndex':
                    dateval = util.ulocalized_time( value,
                            context=self.context,
                            domain='plonelocales',
                            request=self.request)
                    cell.append(dateval)
                elif field['name']=='Title':
                    if result.portal_type in use_view_action:
                        a = u'<a href="%s/view">%s %s</a>'
                    else:
                        a = u'<a href="%s">%s %s</a>'
                    cell.append(a % (result.getURL(), icon_snp, value))
                elif type(value)==tuple:
                    cell.append(', '.join(getattr(result, field['name'])))
                else:
                    cell.append(value)

            json_result['rows'].append(
                {"id":result.getId,"cell":cell})

        self.request.RESPONSE.setHeader('Content-Type','application/json; charset=utf-8')
        return json.dumps(json_result)


class FlexiJsonViewNG(FlexiJsonView):
    """
    FlexiJson browser view
    """
    def _get_search_results(self):
        return get_search_results_ng(self)['results']




from DateTime import DateTime
from zope.component import getUtility
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from plone.app.layout.viewlets import common as base

from Products.CMFCore.utils import getToolByName

from plone.registry.interfaces import IRegistry

from collective.flexitopic.interfaces import IFlexiTopicSettings
from collective.flexitopic import flexitopicMessageFactory as _
from utils import get_search_results, get_topic_table_fields
from utils import IDX_METADATA, DATERANGE_OPERATORS
from utils import get_start_end, get_renderd_table
from utils import get_search_results_ng, get_start_end_ng

try:
    from plone.app.querystring import queryparser
    from plone.app.querystring.registryreader import QuerystringRegistryReader
except ImportError:
    pass

KEYWORD_DELIMITER = ':'
DATE_FIELD_WIDTH = 80
FG_PADDING_WIDTH = 5





class BaseViewlet(base.ViewletBase):
    ''' a common base for the viewlets used here '''

    @property
    def portal_catalog(self):
        return getToolByName(self.context, 'portal_catalog')

class AboutViewlet(BaseViewlet):
    ''' displays the description of a topic '''

class SubtopicViewlet(BaseViewlet):
    ''' displays the subtopics of a topic '''

    def render_table(self, search_results):
        return get_renderd_table(self, search_results)

    def get_subtopics(self):
        stl = []
        if self.context.hasSubtopics():
            for sub_topic in self.context.listSubtopics():
                fields = get_topic_table_fields(sub_topic, self.portal_catalog)
                results = sub_topic.queryCatalog()
                title = sub_topic.Title()
                description = sub_topic.Description()
                text = sub_topic.getText()
                size = sub_topic.getItemCount()
                id = sub_topic.id
                stl.append({'fields': fields,
                            'results': results,
                            'title': title,
                            'description': description,
                            'text': text,
                            'size': size,
                            'start':0,
                            'display_legend': False,
                            'id': 'topic-' + id})

        return stl

class FormViewlet(BaseViewlet):
    ''' displays the query form for old style collections'''

    jslider_template = u'''
                <span id="search-%(id)s">
                    <input type=text"
                        size="10"
                        name="start-%(name)s"
                        id="start-search-%(id)s"
                        value="%(startval)s" />
                        -
                    <input type=text"
                        size="10"
                        name="end-%(name)s"
                        id="end-search-%(id)s"
                        value="%(endval)s" />
                    <input id="slider-%(id)s"
                        type="slider"
                        style="display:none;"
                        name="range-%(name)s"
                        value="%(valrange)s" />

                    <script type="text/javascript" charset="utf-8">
                      jQuery("#slider-%(id)s").slider({
                            from: 0,
                            to: %(to)i,
                            %(calculate)s,
                            %(callback)s
                            });
                    </script>

                </span>
            '''

    @property
    def portal_catalog(self):
        return getToolByName(self.context, 'portal_catalog')

    def _get_calculate_js(self, start_date):
        js_ctemplate = u'''
                calculate: function( value ){
                    var start_date = new Date('%(start)s');
                    var the_date = datesliderhelper.add_date(start_date, value);
                    return datesliderhelper.format_date(the_date);
                }'''
        return js_ctemplate % { 'start': start_date.strftime('%Y/%m/%d')}

    def _get_callback_js(self, name, start_date):
        js_cbtemplate = u'''
                callback: function( value ){
                    var values = value.split(';');
                    var start_date = new Date('%(start)s');
                    var start = datesliderhelper.add_date(start_date, values[0]);
                    var end = datesliderhelper.add_date(start_date, values[1]);
                    $('#start-search-%(name)s').val(datesliderhelper.format_date(start));
                    $('#end-search-%(name)s').val(datesliderhelper.format_date(end));
                    $('#flexitopicresults').flexOptions({newp: 1}).flexReload();
                }'''
        return js_cbtemplate % { 'start': start_date.strftime('%Y/%m/%d'),
                                 'name': name
                                }


    def _sel(self, item, selected):
        if item == selected:
            return u'selected="selected"'
        else:
            return u''

    def _get_index_values(self, idx):
        items = list(self.portal_catalog.Indexes[
                idx].uniqueValues())
        selected = self.request.get(idx,None)
        items.sort()
        item_list =  [{u'name': _(u'All'), u'value':u'',
                u'disabled':None,
                u'selected':self._sel(None,selected)}]
        for item in items:
            item_list.append({u'name': item.decode('utf-8'),
                    u'value':item.decode('utf-8'),
                    u'disabled': None,
                    u'selected':self._sel(item,selected)})
        return item_list


    def _daterange_criterion(self, field_name, start_date, form_start, end_date, form_end):
        date_diff = int(end_date - start_date) + 1
        i_start = int(DateTime(form_start) - start_date)
        i_end = int(DateTime(form_end) - start_date)
        valrange = u'%i;%i' % (i_start, i_end)
        calc_template = self._get_calculate_js(start_date)
        cb_template = self._get_callback_js(field_name,
                    start_date)
        return  self.jslider_template % {
                'id': field_name,
                'name': field_name,
                'startval' : form_start,
                'endval': form_end,
                'valrange': valrange,
                'to': date_diff,
                'calculate': calc_template,
                'callback': cb_template}

    def get_criteria(self):
        ''' combine request with topic criteria '''
        criteria = []
        portal_atct = getToolByName(self.context,'portal_atct')
        for criterion in self.context.listCriteria():
            if criterion.meta_type in ['ATSimpleStringCriterion',
                'ATSelectionCriterion', 'ATListCriterion',
                'ATDateRangeCriterion', 'ATFriendlyDateCriteria']:
                index = portal_atct.getIndex(criterion.Field())
                criterion_field = {'id': criterion.id,
                    'description': index.description,
                    'label': index.friendlyName,
                    'field': criterion.Field(),
                    'type': criterion.meta_type,
                    }
                if criterion.meta_type=='ATSimpleStringCriterion':
                    value = self.request.get(criterion.Field(),'')
                    criterion_field['input'] = u'''<input type="text"
                            size="25"
                            name="%s"
                            id="search-%s"
                            value="%s"/>''' % (criterion.Field(),
                                criterion.Field(), value)
                elif criterion.meta_type in ['ATSelectionCriterion',
                                            'ATListCriterion']:
                    options = u''
                    if criterion.Value():
                        if len(criterion.Value()) ==1:
                            continue
                        selected = self.request.get(criterion.Field(),None)
                        if criterion.getOperator()=='or':
                            # we let the user choose from the selected
                            # values
                            if selected:
                                options = u'<option value="">All</option>'
                            else:
                                options = u'<option selected="selected" value="">All</option>'
                            idx_values = list(criterion.Value())
                            idx_values.sort()
                            for idx_value in idx_values:
                                if idx_value.find(KEYWORD_DELIMITER) > 0:
                                    idx_name=idx_value.split(KEYWORD_DELIMITER)[1]
                                else:
                                    idx_name=idx_value
                                is_selected = self._sel(idx_value,selected)
                                options += u'<option value="%(value)s" %(selected)s >%(name)s</option>' % {
                                    'value': idx_value,
                                    'selected': is_selected,
                                    'name': idx_name}
                            criterion_field['input'] = u'''
                                    <select id="%s" name="%s">
                                    %s
                                    </select>''' % ( criterion.Field(),
                                        criterion.Field(), options)

                        else:
                            # we let the user choose from all possible
                            # values minus the selected ones (as they will
                            # be added (AND) to the search anyway)
                            idx_values = self._get_index_values(criterion.Field())
                            for idx_value in idx_values:
                                if idx_value['value'] in criterion.Value():
                                    continue
                                else:
                                    options += u'<option value="%(value)s" %(selected)s >%(name)s</option>' % idx_value
                            criterion_field['input'] = u'''
                                <select id="%s" name="%s">
                                %s
                                </select>''' % ( criterion.Field(),
                                    criterion.Field(), options)
                    else:
                        # if nothing is selected to search we assume
                        # that we should present a selection with all
                        # posible values
                        idx_values = self._get_index_values(criterion.Field())
                        for idx_value in idx_values:
                            options += u'<option value="%(value)s" %(selected)s >%(name)s</option>' % idx_value
                        criterion_field['input'] = u'''
                            <select id="%s" name="%s">
                            %s
                            </select>''' % ( criterion.Field(),
                                criterion.Field(), options)
                elif criterion.meta_type in ['ATDateRangeCriterion',
                                            'ATFriendlyDateCriteria']:
                    # convert freindly date criteria into date ranges
                    # so we can display a slider to drill down inside
                    # the range
                    start_date, end_date = get_start_end(self, criterion,
                                                self.portal_catalog)
                    startval = self.request.get('start-' + criterion.Field(),
                            start_date.strftime('%Y/%m/%d'))
                    endval = self.request.get('end-' + criterion.Field(),
                            end_date.strftime('%Y/%m/%d'))

                    criterion_field['input'] = self._daterange_criterion(
                        criterion.Field(), start_date, startval,
                        end_date, endval)

                    #criterion_field['input'] = daterange_criterion(criterion)
                else:
                    criterion_field['input'] = None
                criteria.append(criterion_field)
        return criteria


class FormViewletNG(FormViewlet):
    ''' displays the query form for new style collections'''

    def get_criteria(self):
        ''' combine request with topic criteria '''
        criteria = []
        query = queryparser.parseFormquery(self.context, self.context.getRawQuery())
        registry = getUtility(IRegistry)
        qsrr = QuerystringRegistryReader(registry)
        for raw_query in self.context.getRawQuery():
            criterion = qsrr()['indexes'][raw_query['i']]
            input_html = ''
            selected = self.request.get(raw_query['i'],None)
            id = raw_query['i'] + '_' + raw_query['o'].replace('.','_')
            if raw_query['o'] == 'plone.app.querystring.operation.selection.is':
                if len(raw_query['v'])==1:
                    continue
                if selected:
                    options = u'<option value="">All</option>'
                else:
                    options = u'<option selected="selected" value="">All</option>'
                for value in raw_query['v']:
                    is_selected = self._sel(value,selected)
                    optstr = u'<option value="%(value)s" %(selected)s >%(name)s</option>'
                    options += optstr % {
                                        'value': value,
                                        'selected': is_selected,
                                        'name': value}
                    input_html = u'''
                        <select id="%s" name="%s">
                        %s
                        </select>''' % ( id,
                            raw_query['i'], options)
            elif raw_query['o']=='plone.app.querystring.operation.string.contains':
                value = self.request.get(raw_query['i'],'')
                input_html = u'''<input type="text"
                            size="25"
                            id="search-%s"
                            name="%s"
                            value="%s"/>''' % (id,
                                raw_query['i'], value)
            elif raw_query['o'] in DATERANGE_OPERATORS:
                start_date, end_date = get_start_end_ng(self.context, query,
                                                raw_query,
                                                self.portal_catalog)
                startval = self.request.get('start-' + raw_query['i'],
                            start_date.strftime('%Y/%m/%d'))
                endval = self.request.get('end-' + raw_query['i'],
                            end_date.strftime('%Y/%m/%d'))
                input_html = self._daterange_criterion(
                        raw_query['i'], start_date, startval,
                        end_date, endval)

            elif raw_query['o']=='plone.app.querystring.operation.date.largerThanRelativeDate':
                #within last n days
                pass
            elif raw_query['o']=='plone.app.querystring.operation.date.beforeToday':
                pass
            elif raw_query['o']=='plone.app.querystring.operation.date.lessThanRelativeDate':
                #within next n days
                pass
            elif raw_query['o']=='plone.app.querystring.operation.date.largerThan':
                #after date
                import ipdb; ipdb.set_trace()
            elif raw_query['o']=='plone.app.querystring.operation.date.today':
                pass
            elif raw_query['o']=='plone.app.querystring.operation.date.afterToday':
                pass
            elif raw_query['o']=='plone.app.querystring.operation.date.lessThan':
                # before date
                pass
            elif raw_query['o']=='plone.app.querystring.operation.date.between':
                # between date and date
                pass
            else:
                continue

            criterion_field = {
                'id': raw_query['i'] + '_' + raw_query['o'].replace('.','_'),
                'description': criterion['description'],
                'label': criterion['title'],
                'field': raw_query['i'],
                'input': input_html,
                }
            criteria.append(criterion_field)
        return criteria

class FlexigridViewlet(BaseViewlet):
    ''' displays the flexigrid results'''

class ResultTableViewlet(BaseViewlet):
    '''plain html results table '''


    def get_table_fields(self):
        return get_topic_table_fields(self.context, self.portal_catalog)

    def render_table(self, search_results):
        return get_renderd_table(self, search_results)

    def search_results(self):
        results = get_search_results(self)
        results['fields'] = self.get_table_fields()
        results['id'] = "topichtmlresults"
        results['display_legend'] = (results['num_results'] > 0)
        return results

class ResultTableViewletNG(ResultTableViewlet):

    def search_results(self):
        results = get_search_results_ng(self)
        results['fields'] = self.get_table_fields()
        results['id'] = "topichtmlresults"
        results['display_legend'] = (results['num_results'] > 0)
        return results



class JsViewlet(BaseViewlet):
    ''' inserts the js to render the above viewlets '''


    render = ViewPageTemplateFile("templates/jstemplate.pt")


    js_template = """
 $(document).ready(function() {
   $('#flexitopicsearchform').find('select').each(function(i) {
     $(this).change(function( objEvent ){
         $('#flexitopicresults').flexOptions({newp: 1}).flexReload();
        }) ;
   });
   $('#flexitopicsearchform').find('input:radio').each(function(i) {
     $(this).change(function( objEvent ){
         $('#flexitopicresults').flexOptions({newp: 1}).flexReload();
        }) ;
   });
 });

    $("#flexitopicresults").flexigrid
            (
            {
            url: '%(url)s',
            dataType: 'json',
            colModel : [
                %(col_model)s
                ],
            %(sort)s
            usepager: true,
            title: '%(title)s',
            useRp: true,
            rp: %(items_ppage)i,
            showTableToggleBtn: false,
            width: %(width)i,
            onSubmit: addFormData,
            height: %(height)i
            }
            );

    function addFormData() {
        var dt = $('#flexitopicsearchform').serializeArray();
        $("#flexitopicresults").flexOptions({params: dt});
        %(add_js)s;
        return true;
    };

    $('#flexitopicsearchform').submit(function (){
                $('#flexitopicresults').flexOptions({newp: 1}).flexReload();
                return false;
            }
    );


        """

    add_form_data_js ='//%s'


    def get_js(self):
        """{display: 'Title', name : 'Title', width : 220, sortable : true, align: 'left'}"""
        def is_sortable(sortname):
            if sortname in IDX_METADATA.keys():
                return True
            elif sortname in self.portal_catalog.Indexes.keys():
                if self.portal_catalog.Indexes[sortname].meta_type in [
                        'FieldIndex', 'DateIndex', 'KeywordIndex']:
                    return True
            return False
        def is_date(sortname):
            if sortname in IDX_METADATA and sortname !='Title':
                return True
            elif sortname in self.portal_catalog.Indexes.keys():
                if self.portal_catalog.Indexes[sortname].meta_type in ['DateIndex']:
                    return True
            return False
        fields = get_topic_table_fields(self.context, self.portal_catalog)
        registry = getUtility(IRegistry)
        settings = registry.forInterface(IFlexiTopicSettings)
        width=settings.flexitopic_width
        height=settings.flexitopic_height
        i_date = 0
        for field in fields:
            if is_date(field['name']):
                i_date += 1
        date_fields_width = DATE_FIELD_WIDTH * i_date
        if len(fields) > i_date:
            field_width=int(
                (width - date_fields_width - 18 -
                    (len(fields) * FG_PADDING_WIDTH))/
                (len(fields)-i_date))
        else:
            field_width = DATE_FIELD_WIDTH
        t = "{display: '%s', name : '%s', width : %i, sortable : %s, align: 'left', hide: false}"
        tl = []
        for field in fields:
            this_field_width = 0
            if is_date(field['name']):
                this_field_width = DATE_FIELD_WIDTH
            else:
                this_field_width = field_width
            if is_sortable(field['name']):
                sortable='true'
            else:
                sortable='false'
            tl.append( t % (self.context.translate(_(field['label'])),
                        field['name'], this_field_width, sortable))
        sort = ''
        if hasattr(self.context, 'listCriteria'):
            #old style collection
            for criterion in self.context.listCriteria():
                if criterion.meta_type =='ATSortCriterion':
                    sortname = criterion.getCriteriaItems()[0][1]
                    sortorder = 'asc'
                    if len(criterion.getCriteriaItems())==2:
                        if criterion.getCriteriaItems()[1][1] =='reverse':
                            sortorder = 'desc'
                    sort = "sortname: '%s', sortorder: '%s'," % (
                                sortname, sortorder)
        elif hasattr(self.context, 'getSort_on'):
            #new style collection
            sortname = self.context.getSort_on()
            sortorder = 'asc'
            if self.context.getSort_reversed():
                sortorder = 'desc'
            else:
                sortorder = 'asc'
            sort = "sortname: '%s', sortorder: '%s'," % (
                                sortname, sortorder)
        table_name = self.context.Title()
        url = self.context.absolute_url() + '/@@flexijson_view'
        try:
            items_ppage = self.context.getItemCount()
        except AttributeError:
            items_ppage = settings.items_pp
        add_form_data_js = self.add_form_data_js % self.context.absolute_url()
        if items_ppage==0:
            items_ppage = settings.items_pp
        js = self.js_template % {
                'url': url,
                'col_model': ', '.join(tl),
                'sort': sort,
                'title': table_name,
                'items_ppage': items_ppage,
                'add_js': add_form_data_js,
                'width': width,
                'height': height,
            }
        return js




from setuptools import setup, find_packages
import os

version = '0.6'

setup(name='collective.flexitopic',
      version=version,
      description="Searchable Collections with Flexigrid",
      long_description=open("README.rst").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      # Get more strings from
      # http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Framework :: Plone :: 4.0",
        "Framework :: Plone :: 4.1",
        "Framework :: Plone :: 4.2",
        "Framework :: Plone :: 4.3",
        "Programming Language :: Python",
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        ],
      keywords='',
      author='Christian Ledermann',
      author_email='christian.ledermann@gmail.com',
      url='https://github.com/collective/collective.flexitopic',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['collective'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
          'plone.app.registry',
      ],
      entry_points="""
      # -*- Entry points: -*-

      [z3c.autoinclude.plugin]
      target = plone
      """,
      )
